%% UniqueNotify.lhs

%% Copyright (c) 2007 by Conal Elliott
%% Created: Fri Jun 01, 2007 11:58
%% Last Modified: Sun Jul 29, 2007 22:26 by Conal Elliott

\section{UIs with unique notfication} \label{sec:UniqueUI}

%if False
\begin{code}
{-# OPTIONS -fglasgow-exts #-}
module UniqueNotify where

import Data.Monoid
import Control.Monad (join)
import Control.Arrow (first,second)
import Control.Applicative hiding (empty)
-- import Foreign (unsafePerformIO)

import Graphics.UI.WX

import O
import DataDriven
import SimpleUI (above, leftOf, Unop, Win, hwidget)
import FlexibleLayout (MonoidDict,CxMonoid(..),CxLayout,widgetCXL)
import qualified FlexibleLayout as F
\end{code}
%endif

\emph{Note:} Wolfgang Jeltsch pointed out that this optimization described in this section is not necessary.\footnote{\talkpage}
The reason is that sources are never accessible to clients of the |UI| or |UI'| types, and the abstraction never replicates the sources it creates.
Thus the problem I'm trying to avoid cannot happen anyway.
So, I don't recommend reading this section.

As an optimization, we next switch to notifier representation in \secref{UNotifier} for non-redundant notification.
Relative to \secref{FlexibleLayout}, the new |UI| representation adds a means of generating unique tags and uses |USource| in place of |Source|.
\begin{code}
type UI' a = NewTag -> Win -> IO (CxLayout, USource a)

type UI =
  (->) NewTag `O` (->) Win `O` IO `O` (,) CxLayout `O` USource
\end{code}
%if False
\begin{code}
ui    :: UI'  a -> UI   a
unUI  :: UI   a -> UI'  a

ui    = O . O . O . O
unUI  = unO . unO . unO . unO
\end{code} %$
%endif

Running a UI works as in \secref{FlexibleLayout}, except that |newNewTag| (\secref{UNotifier}) is invoked to make a tag generator to pass in.
%if False
\begin{code}
runUI :: UI (IO ()) -> IO ()
runUI = runNamedUI "Phooey GUI"
\end{code} %$
%endif
\begin{code}
runNamedUI :: String -> UI (IO ()) -> IO ()
runNamedUI name ui = start $
  do  f           <- frame [ visible := False, text := name ]
      newTag      <- newNewTag
      win         <- panel f []
      (cxl,msrc)  <- unUI ui newTag win
      set win  [ layout := unCxMonoid cxl (empty,above) ]
      set f    [ layout := hwidget win, visible := True ]
      runDD (fromUSource msrc)
\end{code} %$

%% Widgets
%if False
\begin{code}
type IWidget  a = a -> UI a
type OWidget  a = UI (a -> IO ())

type MkWidget wid a b =
  (Win -> [Prop wid] -> IO wid) -> Attr wid a -> b

stringEntry :: IWidget String
stringEntry = iwidget textEntry text

islider :: (Int,Int) -> IWidget Int
islider (lo,hi) =
  iwidget (\ win -> hslider win True lo hi) selection

stringDisplay :: OWidget String
stringDisplay = owidget textEntry text

showDisplay :: Show a => OWidget a
showDisplay = fmap (. show) stringDisplay

iwidget :: (Commanding wid, Widget wid) =>
   MkWidget wid a (IWidget a)

owidget :: Widget wid => MkWidget wid a (OWidget a)
owidget mkWid attr = ui $ \ _ win ->
  do  wid <- mkWid win [ ]
      return (widgetCXL wid, pure (\ a -> set wid [ attr := a ]))
\end{code} %$
%endif

The only changes in widget creation use (a) use of the passed in tag generator to make a unique tag and (b) conversion to an |USource|.
\begin{code}
iwidget mkWid attr initial = ui $ \ newTag win ->
  do  wid  <- mkWid win [ attr := initial ]
      tag  <- newTag
      src  <- fmap (toUSource tag) (attrSource wid attr)
      return (widgetCXL wid, src)
\end{code} %$

%if False
Everthing else is unchanged.
\begin{code}
leftToRight, topToBottom, flipLayout :: Unop (UI a)
leftToRight  = withCxMonoid (empty,leftOf)
topToBottom  = withCxMonoid (empty,above)
flipLayout   = compCxMonoid (second flip)

withCxMonoid :: MonoidDict Layout -> Unop (UI a)
withCxMonoid op = compCxMonoid (const op)

compCxMonoid :: Unop (MonoidDict Layout) -> Unop (UI a)
compCxMonoid f = onCxLayout' (. f)

onCxLayout :: Unop CxLayout -> Unop (UI a)
onCxLayout f = ui . (fmap.fmap.fmap.first) f . unUI

onCxLayout' :: Unop (MonoidDict Layout -> Layout) -> Unop (UI a)
onCxLayout' f' = onCxLayout (CxMonoid . f' . unCxMonoid)

title :: String -> Unop (UI a)
title str = onCxLayout' (boxed str .)


apples, bananas, fruit :: UI Int
apples   = title "apples"   $ islider (0,10) 3
bananas  = title "bananas"  $ islider (0,10) 7
fruit    = title "fruit"    $ liftA2 (+) apples bananas

total :: Show a => OWidget a
total = title "total" showDisplay

shopping :: UI (IO ())
shopping = title "Shopping List" $ fruit <**> total

shoppingFlip  = flipLayout shopping
shoppingLR    = leftToRight shopping
shoppingTLR   = leftToRight fruit <**> total
\end{code}
%endif
